% This program is abble to estimate the discharge from tracer time series
% This code is divided in several parts:
%   - file uploads
%   - good mixing length checks
%   - optimization of the couple (Concentrqtion, Volume) to minimise
%   uncertainties)
%   - calibration correction from the oupt file of rw123etalo.m
%   - Signal analysis
%   - discharge calculation ans its uncertainty
%   - confidence interval calculation

% Wrote by Mathieu Lepot in 2010 (INSA Lyon, Sepia Conseils)
% Adapted in 2021 for the IWA book
% Present address: Un Poids Une Mesure, mathieu.lepot@upum.fr

% This code was designed for the fluorometer SCUFA from Turner Design
% It can be easily adapted for other fluorometer nor sensors; with tiny
% changes

% To run this code, you need the following files:
%   - av_1, pic_1 and respectively ap_1.txt containing time series before,
%   during and after the peak
%   - D_10-100 and D_100-1000.txt with volumes and uncertqinties of the tested
%   injection devices
%   - dilution or sans_dilution.txt with concentrations and their
%   uncertainties for injection solutions
%   - regw123etalo.dat with calibration results of the fluorometer
% THOSE ARE FILES YOU CAN USE TO TRAIN OURSELF WITH THE CODE. YOU HQVE TO
% CONSTRUCTION NEW FILES WITH YOUR DATA, SOLUTIONS AND SENSORS IN ORDER TO
% OBTAIN RELIABLE RESULTS WITH YOUR TRACERÒ EXPERIMENTS

clear all
clf
clc

%___________________________________________________________________________
% CALCULATION OF Minj, THE MASS THAT HAS O BE INJECTED
 
x0=input('Distance (in m) between injection and measuring locations: ');
h0=input('Water level (in m): : ');
u0=input('Velocity (in m/s): ');
I=input('Pipe slope (in m/m): ');
s0=input('Wet section (in m^2): ');
dt=input('Acquisition time step (in sec): ');
u_Cmes=input('Standard uncertainty of the recorded concentrations (in ppb): ');



geom=input('Is the reach homogenous from injection to measurement locations (yes or no)? ','s');

if geom=='yes'
    k0=6*h0*sqrt(9.81*h0*I);
else
    if geom=='no'
        p0=input('Wet perimeter (in m): ');
        W0=input('Width at the free surface (in m): ');
        Rh0=s0/p0;
        H0=s0/W0;
        ut0=sqrt(9.81*Rh0*I);
        k0=5.92*(u0/ut0)^1.43*(W0/H0)^0.62*H0*ut0;
    end
end

d0=[x0 k0 u0 s0];

%__________________________________________________________________________
% VERIFICATION OF THE GOOD MIXING LENGTH
a=x0/h0;
    if a < 75
        warning('The reach is too short');
    else
        if a > 150
        disp('The reach is long enough for injection at the side');
        else 
            disp('The reach is long enough for injection in the middle of the flow');
        end
    end

clear a;

%__________________________________________________________________________
% OPTIMSATION OF(CONCENTRATION, VOLUME)

    % Upload of the repeatability tests of the pipettes
    
    V=load('D_10-100.txt');
    B=load('D_100-1000.txt');
    [nV,mV]=size(V);
    [nB,mB]=size(B);
    
    if V(nV,2)>B(1,2)
        for i=nV:nV+nB-1
            V(i,:)=B(nV-i+1,:);
        end
    else
        for i=nV+1:nV+nB-1
            V(i,:)=B(i+1-nV,:);
        end
    end
    
    % Upload of concetration uncertainty file
    
    Z=load('sans_dilution.txt');
    
    % Calculation of the expected trqcer time series at the measurement
    % location
    
    t=[1:dt:3600];
    cmax=input('Target value of the peak concentration at the measurement location (in ppb - typically 80): ');
    
    for i=1:length(t)
            C(1,i)=1/(s0*sqrt(4*pi*k0*t(i)))*exp(-((x0-u0*t(i))^2)/(4*k0*t(i))); %en ppb
    end

    tcmax=dt*find(C==max(C));
    
    clear  B & nV & mV & nB & mB & Dil & t;   

    % Optimisation of (Concentration, Volume)
    
    Con=Z(:,1);
    u_Con=Z(:,2);
    Vol=V(:,1);
    u_Vol=V(:,2);
    M=10^(-12)*Z(:,1)*V(:,1)';
    [n,m]=size(M);
    for i=1:n
        for j=1:m
            ECmax(i,j)=abs(10^3*M(i,j)/(s0*sqrt(4*pi*k0*tcmax))*exp(-((x0-u0*tcmax)^2)/(4*k0*tcmax))-cmax);
        end
    end
    
    Vol_Lu=[10 ; 20 ; 30 ; 40 ; 50 ; 60 ; 70 ; 80 ; 90 ; 100 ; 200 ; 300 ; 400 ; 500 ; 600 ; 700 ; 800 ; 900 ; 1000];
        
    
    [i j]=find(ECmax < 30);
    L=length(i);
    
    for k=1:L
        C(k,1)=Con(i(k));
        C(k,2)=u_Con(i(k));
        C(k,3)=Vol(j(k));
        C(k,4)=u_Vol(j(k));
        C(k,5)=10^(-12)*C(k,1)*C(k,3);
        C(k,6)=sqrt((10^-3*C(k,2))^2*(10^-9*C(k,3))^2+(10^-9*C(k,4))^2*(10^-3*C(k,1))^2);
    end
    
    ind=find(C(:,6)==min(C(:,6)));
    
    %c0, v0, m0, u_Cinj et u_Vinj
    
    c0=C(ind,1);
    u_Cinj=C(ind,2);
    v0=C(ind,3);
    u_Vinj=C(ind,4);
    vol_lu=Vol_Lu(j(k));
    m0=C(ind,5);
    cmax0=10^3*m0/(s0*sqrt(4*pi*k0*tcmax))*exp(-((x0-u0*tcmax)^2)/(4*k0*tcmax));
    
    if c0==213300000
         disp(['In order to get the lowest uncertainties, you should inject ',num2str(vol_lu),' micro-L of the commercial solution.']);
    else
        if c0==21330000
           disp(['In order to get the lowest uncertainties, you should inject ',num2str(vol_lu),' micro-L of the commercial solution, diluted by a factor of 10.']);
        else
           if c0==2133000
               disp(['In order to get the lowest uncertainties, you should inject ',num2str(vol_lu),' micro-L of the commercial solution, diluted by a factor of 100.']);
           else
               disp(['In order to get the lowest uncertainties, you should inject ',num2str(vol_lu),' micro-L of the commercial solution, diluted by a factor of 1000.']);
           end
        end
    end
   
    disp(['The peak concentration should be arround: ',num2str(cmax0),' ppb.']);
    
clear Con & u_Con & Vol & u_Vol & V & Z & ECmax & i & j & C & ind & L & m & n & cmax0;    

%___________________________________________________________________________
% UPLOAD TO THE CONCENTRATION TIME SERIES - Craw
Nomrhodamine=input('Name of the .txt file of concentration time series (do not write the extension): ','s');
[A1,A2,A3,A4]=textread([Nomrhodamine,'.txt'],'%10c %8c %f %f','commentstyle','matlab','headerlines',1,'delimiter',';');
B=strcat(A1,(' '),A2);
date=datenum(B, 'dd/mm/yyyyHH:MM:SS');
A=[date A3 A4];
A(1,:)=[];
n=length(A);

clear date & B;

%___________________________________________________________________________
% CALIBRATION CORRECTION

etal=input('Name of the .dat file of the calibration correction (do not write the extension): ','s');
E=load([etal,'.dat']);
degre=input('Select the order of the calibration function: 1, 2 or 3: ','s');

    %----------------------------------------------------------------------
    % Calculations of Ccorr and their strandard uncertainties

if degre=='1'
    Y=E(1:7,:);
    P=[Y(3) Y(2)];
        for i=1:length(A)
        	A(i,4)=(A(i,2)-Y(2))/Y(3);
        	A(i,5)=sqrt(1/(P(1)^2)*(u_Cmes^2+Y(4).^2+Y(5).^2*A(i,4).^2+2*A(i,4)*Y(6)));
        end
else
    if degre=='2'
       Y=E(8:18,:);
       P=[Y(4) Y(3) Y(2)];
            for i=1:length(A)
                Q(i,:)=[Y(4) Y(3) Y(2)-A(i,2)];
                rac(i,:)=roots(Q(i,:));
                for l=1:2
                    if imag(rac(i,l))~=0
                        rac(i,l)=NaN;
                    end
                end
                dis(i,:)=abs(rac(i,:)-A(i,2));
                m_in(i)=find(dis(i,:)==min(dis(i,:)));
                A(i,4)=rac(i,m_in(i));
                if isnan(A(i,4)) == 1
                    A(i,5)=NaN;
                else
                    A(i,5)=sqrt(1/((P(2)+2*P(1)*A(i,4))^2)*(u_Cmes^2*+Y(5).^2+Y(6).^2*A(i,4).^2+Y(7).^2*A(i,4).^4+2*A(i,4)*Y(8)+2*A(i,4).^2*Y(9)+2*A(i,4).^3*Y(10)));
                end
            end
    else
        if degre=='3'
        Y=E(19:34,:);
        P=[Y(5) Y(4) Y(3) Y(2)];
            for i=1:length(A)
                Q(i,:)=[Y(5) Y(4) Y(3) Y(2)-A(i,2)];
                rac(i,:)=roots(Q(i,:));
                for l=1:3
                    if imag(rac(i,l))~=0
                        rac(i,l)=NaN;
                    end
                end
                dis(i,:)=abs(rac(i,:)-A(i,2));
                m_in(i)=find(dis(i,:)==min(dis(i,:)));
                A(i,4)=rac(i,m_in(i));
                if A(i,4)==NaN
                    A(i,5)=NaN;
                else
                    A(i,5)=sqrt(1/((P(3)+2*P(2)*A(i,4)+3*P(1)*A(i,4).^2)^2)*(u_Cmes^2+Y(6)^2+Y(7)^2*A(i,4).^2+Y(8)^2*A(i,4).^4+Y(9)*A(i,4).^6+2*A(i,4)*Y(10)+2*A(i,4).^2*Y(11)+2*A(i,4).^3*Y(12)+2*A(i,4).^3*Y(13)+2*A(i,4).^4*Y(14)+2*A(i,4).^5*Y(15)));
                end
            end
        end
    end
    
end

clear etal & Q & rac & dis & m_in;

%__________________________________________________________________________
% SIGNAL ANALYSIS
    
    %----------------------------------------------------------------------
    % Background noise
    
    Bruitavant=input('Name of the .txt file of concetration time series from the background noise before the peak (do not write the extension): ','s');
    [B1,B2,B3,B4]=textread([Bruitavant,'.txt'],'%10c %8c %f %f','commentstyle','matlab','headerlines',1,'delimiter',';');
    C=strcat(B1,(' '),B2);
    date=datenum(C, 'dd/mm/yyyyHH:MM:SS');
    B=[date B3 B4];
    B(1,:)=[];
    b=length(B);
    for i=1:b
        if degre=='1'
                for i=1:b
                    B(i,4)=(B(i,2)-Y(2))/Y(3);
                    B(i,5)=sqrt(1/(P(1)^2)*(u_Cmes^2+Y(4).^2+Y(5).^2*B(i,4).^2+2*B(i,4)*Y(6)));
                end
        else
            if degre=='2'
                    for i=1:b
                        P(i,:)=[Y(4) Y(3) Y(2)-B(i,2)];
                        rac(i,:)=roots(P(i,:));
                        for l=1:2
                            if imag(rac(i,l))~=0
                                rac(i,l)=NaN;
                            end
                        end
                        dis(i,:)=abs(rac(i,:)-B(i,2));
                        m_in(i)=find(dis(i,:)==min(dis(i,:)));
                        B(i,4)=rac(i,m_in(i));
                        if B(i,4)==NaN
                            B(i,5)=NaN;
                        else
                            B(i,5)=sqrt(1/((P(2)+2*P(1)*B(i,4))^2)*(u_Cmes^2+Y(5).^2+Y(6).^2*B(i,4).^2+Y(7).^2*B(i,4).^4+2*B(i,4)*Y(8)+2*B(i,4).^2*Y(9)+2*B(i,4).^3*Y(10)));
                        end
                    end
            else
                if degre=='3'
                Y=E(19:34,:);
                    for i=1:b
                        P(i,:)=[Y(5) Y(4) Y(3) Y(2)-B(i,2)];
                        rac(i,:)=roots(P(i,:));
                        for l=1:3
                            if imag(rac(i,l))~=0
                                rac(i,l)=NaN;
                            end
                        end
                        dis(i,:)=abs(rac(i,:)-B(i,2));
                        m_in(i)=find(dis(i,:)==min(dis(i,:)));
                        B(i,4)=rac(i,m_in(i));
                        if B(i,4)==NaN
                            B(i,5)=NaN;
                        else
                            B(i,5)=sqrt(1/((P(3)+2*P(2)*B(i,4)+3*P(1)*B(i,4).^2)^2)*(u_Cmes^2+Y(6)^2+Y(7)^2*B(i,4).^2+Y(8)^2*B(i,4).^4+Y(9)*B(i,4).^6+2*B(i,4)*Y(10)+2*B(i,4).^2*Y(11)+2*B(i,4).^3*Y(12)+2*B(i,4).^3*Y(13)+2*B(i,4).^4*Y(14)+2*B(i,4).^5*Y(15)));
                        end
                    end
                end
            end
    
        end
        
    end
   
    Cbruit=mean(B(:,4));
    uCbruit=std(B(:,4));
     
clear date & Q & rac & dis & m_in & B & C;
     
    Bruitapres=input('Name of the .txt file of concetration time series from the background noise after the peak (do not write the extension): ','s');
    [C1,C2,C3,C4]=textread([Bruitavant,'.txt'],'%10c %8c %f %f ','commentstyle','matlab','headerlines',1,'delimiter',';');
    D=strcat(C1,(' '),C2);
    date=datenum(D, 'dd/mm/yyyyHH:MM:SS');
    C=[date C3 C4];
    C(1,:)=[];
    c=length(C);
    for i=1:c
        if degre=='1'
                for i=1:c
                    C(i,4)=(C(i,2)-Y(2))/Y(3);
                    C(i,5)=sqrt(1/(P(1)^2)*(u_Cmes^2+Y(4).^2+Y(5).^2*C(i,4).^2+2*C(i,4)*Y(6)));
                end
        else
            if degre=='2'
                    for i=1:c
                        P(i,:)=[Y(4) Y(3) Y(2)-C(i,2)];
                        rac(i,:)=roots(P(i,:));
                        for l=1:2
                            if imag(rac(i,l))~=0
                                rac(i,l)=NaN;
                            end
                        end
                        dis(i,:)=abs(rac(i,:)-C(i,2));
                        m_in(i)=find(dis(i,:)==min(dis(i,:)));
                        C(i,4)=rac(i,m_in(i));
                        if C(i,4)==NaN
                            C(i,5)=NaN;
                        else
                            C(i,5)=sqrt(1/((P(2)+2*P(1)*C(i,4))^2)*(u_Cmes^2+Y(5).^2+Y(6).^2*C(i,4).^2+Y(7).^2*C(i,4).^4+2*C(i,4)*Y(8)+2*C(i,4).^2*Y(9)+2*C(i,4).^3*Y(10)));
                        end
                    end
            else
                if degre=='3'
                Y=E(19:34,:);
                    for i=1:c
                        P(i,:)=[Y(5) Y(4) Y(3) Y(2)-C(i,2)];
                        rac(i,:)=roots(P(i,:));
                        for l=1:3
                            if imag(rac(i,l))~=0
                                rac(i,l)=NaN;
                            end
                        end
                        dis(i,:)=abs(rac(i,:)-C(i,2));
                        m_in(i)=find(dis(i,:)==min(dis(i,:)));
                        C(i,4)=rac(i,m_in(i));
                        if C(i,4)==NaN
                            C(i,5)=NaN;
                        else
                            C(i,5)=sqrt(1/((P(3)+2*P(2)*C(i,4)+3*P(1)*C(i,4).^2)^2)*(u_Cmes^2+Y(6)^2+Y(7)^2*C(i,4).^2+Y(8)^2*C(i,4).^4+Y(9)*C(i,4).^6+2*C(i,4)*Y(10)+2*C(i,4).^2*Y(11)+2*C(i,4).^3*Y(12)+2*C(i,4).^3*Y(13)+2*C(i,4).^4*Y(14)+2*C(i,4).^5*Y(15)));
                        end
                    end
                end
            end
    
        end
    end
    Cbruitconfirmation=mean(C(:,4));
    uCbruitconfirmation=std(C(:,4));

    ecart=abs(Cbruit-Cbruitconfirmation);
    u_ecart=sqrt(uCbruit^2+uCbruitconfirmation^2);
    
    clear date & Q & rac & dis & m_in & D;
    
    if ecart>2*u_ecart
       warning('Background noise values are not consistent');
       ans=input('Would you like to take the values before "B" or after "A" for further calculations ','s');
        if ans=='B'
            Cbruitf=Cbruit;
            uCbruitf=uCbruit;
        else
            if ans=='A'
                Cbruitf=Cbruitconfirmation;
                uCbruitf=uCbruitconfirmation;
            else
                ans2=input('Please write a manual value of the backgroundnoise (in ppb): ');
                Cbruitf=ans2;
                ans3=input('Please write the standard uncertainty of the backgroundnoise (in ppb): ');
                uCbruitf=ans3;
            end
        end
    else
        disp('Background noise values are consistent');
        Cbruitf=(Cbruit+Cbruitconfirmation)/2;
        uCbruitf=max(uCbruit,uCbruitconfirmation);
    end
    
clear B & C & b & c & P & Y & E & Cbruit & uCbruit & Cbruitconfirmation & uCbruitconfirmation & ecart & u_ecart & ans & ans2 & ans3;

    %----------------------------------------------------------------------
    % Automatic detection of start and end of the peak
    
        %..................................................................
        % Start
    
        for i=1:length(A)
            B(i,1)=A(i,4)-Cbruitf;
            B(i,2)=2*sqrt(A(i,5)^2+uCbruitf^2);
            B(i,3)=B(i,1)-B(i,2);
        end
    
        t=find(B(:,3)>0);

        if isempty(t)==1
            warning('The analysis shows no peak');
        else
            d=t(1);
            td=A(max(1,d-1),1);
            M=A(max(1,d-1):length(A),:);
        end
       
        m=length(M);
    
        clear B & t & r;
    
       %...................................................................
       % End
    
        for i=1:m
            C(i,1)=M(i,4)-Cbruitf;
            C(i,2)=2*sqrt(M(i,5)^2+uCbruitf^2);
            C(i,3)=C(i,1)-C(i,2);
        end
    
        t=find(C(:,3)>0);
        
        if isempty(t)==1
            warning('The record is too short to indetify the end of the peak');
        else
            f=t(length(t));
            tf=M(f,1);
            N=M(1:f,:);
        end

        n=length(N);
    
        clear C & t & r;      

        %.................................................................. 
        % Representation of start and end
        
        g=find(A(:,1)==td);
        Z=A;
        Z(max(g-1,1):length(A),:)=NaN;
        
        e=find(A(:,1)==tf);
        L=A;
        L(1:min(e+1,length(A)),:)=NaN;
        
        
        figure(2)
        plot(A(:,1),A(:,4));
        hold on;
        plot(A(:,1),Z(:,4),'g');
        hold on;
        plot(A(max(1,g-1),1),A(max(g-1,1),4),'+');
        hold on;
        plot(A(min(e+1,length(A)),1),A(min(e+1,length(A)),4),'+');
        hold on;
        plot (A(:,1),L(:,4),'r');
        hold on;
        plot(A(:,1),Cbruitf);
        hold on;
        plot(A(:,1),Cbruitf+2*uCbruitf,'-.');
        hold on;
        plot(A(:,1),Cbruitf-2*uCbruitf,'-.');
        datetick('x',13);
        title('Automatic identification of start and end');
        hgsave(2,[Nomrhodamine, '_Start&End.fig']);
        
        disp(['Start took place at: ',datestr(td)]);
        ansd=input('Would you like to change it (yes or no)? ','s');
        if ansd=='no'
            td=td;
        else
            td_con=input('Please enter the start according to the format (DD/MM/YYYY HH:MM:SS) : ','s');
            td=datenum(td_con,'dd/mm/yyyy HH:MM:SS');
        end

        disp(['End took place at: ',datestr(tf)]);
        ansf=input('Would you like to change it (yes or no)? ','s');
        if ansf=='no'
            tf=tf;
        else
            tf_con=input('Please enter the start according to the format (DD/MM/YYYY HH:MM:SS) : ','s');
            tf=datenum(tf_con,'dd/mm/yyyy HH:MM:SS');
        end

close 2;

clear td_con & tf_con;
        
        %...................................................................
        % Saving of the peak
        
        x=find(A(:,1)==td);
        P=A;
        P(1:x-1,:)=[];
        y=find(P(:,1)==tf);
        p=length(P);
        P(y+1:p,:)=[];
        o=length(P);
        
        for i=1:o
            P(i,6)=P(i,4)-Cbruitf;
            P(i,7)=sqrt(P(i,5)^2+uCbruitf^2);
        end
        
        P(:,4:5)=[];
        save P.dat;
        
clear N & x & y & p;
    
    %----------------------------------------------------------------------
    % Check width the gradient
    
    imax=find(P(:,4)==max(P(:,4)));    
    
        %..................................................................
        % Positive gradient before the max
              
        for i=1:imax-1
            G(i,1)=P(i+1,4)-P(i,4);
        end
        
        conformite_test_grad_ascendant=find(G(:,1)>0);
        valide_G=P(conformite_test_grad_ascendant,:);

        if length(valide_G)==length(G)
            disp('Data are good according to the gradient test on the ascending part');
        
        else
             warning('There is (are) artefact(s) on the ascending part');
             erreur_test_grad_ascendant=find(G<=0);
             erreur_G=P(erreur_test_grad_ascendant,:);
             Z=P;
             Z(erreur_test_grad_ascendant,:)=NaN;
            
             figure(3)
             clf;
             plot(P(1:imax,1),P(1:imax,4),'+b');
             datetick('x',13);
             hold on;
             plot(P(1:imax,1),Z(1:imax,4),'r');
             datetick('x',13);
             title('Artefact(s) on the ascending part');
             hgsave(3,[Nomrhodamine, '_Ascending_part.fig']);
            
            [g1,g2]=size(erreur_G);
            
            h=1;
            
            while h<=g1
                inf_h=erreur_test_grad_ascendant(h)-1;
                sup_h=erreur_test_grad_ascendant(h)+1;
                inf_eventuel=find(conformite_test_grad_ascendant==inf_h);
                sup_eventuel=find(conformite_test_grad_ascendant==sup_h);
                exist_inf_h=isempty(inf_eventuel);
                exist_sup_h=isempty(sup_eventuel);
                
                if exist_inf_h + exist_sup_h==0
                    indice_h=erreur_test_grad_ascendant(h);
                    disp(['For the time stamp: ',datestr(P(indice_h+1,1))]);
                    ans=input('Would you like to take the mean for the missing value (yes or no)? ','s');
                    	if ans=='yes'
                            P(indice_h+1,4)=(P(indice_h+2,4)+P(indice_h,4))/2;
                        else
                            rec=input(['Enter the value here: ']);
                            P(indice_h,4)=rec;
                        end
                
                h=h+1;      
                  
                else
                    warning('The artefact occured for several time steps in a row')
                    indice_h=erreur_test_grad_ascendant(h);
                    G_inf=G(1:indice_h-1,:);
                    conformite_test_grad_ascendant_G_inf=find(G_inf>0);
                    a=max(conformite_test_grad_ascendant_G_inf);
                    G_sup=G(indice_h+1:length(G),:);
                    conformite_test_grad_ascendant_G_sup=find(G_sup>0);
                    b=min(conformite_test_grad_ascendant_G_sup);
                    if G(1)<=0
                        warning('Values are doubtful from the start');
                    else
                        disp(['Data between ',datestr(P(a+1,1)),' & ',datestr(P(indice_h+b+1,1)),' are doubtful']);
                        disp('Please enter the value for the following the stamps: ');
                        for e=a+1:indice_h+b+1
                            disp([datestr(P(e,1)),' : ',num2str(P(e,4))]);
                            val_e=input('Value for the time stamp: ');
                            P(e,4)=val_e;
                        end
                    end
                
                h=h+b+1;
                    
                end
  
            end
            
            close 3;
      
        end                       


      disp('Data are validated on the ascending part');
      
clear G & Z & conformite_test_grad_ascendant & valide_G & erreur_test_grad_ascendant & g1 & g2 & h & inf_h & sup_h & inf_eventuel & sup_eventuel & exist_inf_h & exist_sup_h & indice_h & ans & rec & G_inf & g_inf & conformite_test_grad_ascendant_inf & a & G_sup & g_sup & conformite_test_grad_ascendant_sup & b & e & val_e; 
   
        %..................................................................
        % Negaive gradient after the max
   
        for i=imax:o-1
            F(i,1)=P(i+1,4)-P(i,4);
        end
        
        F(1:imax-1,:)=[];

        conformite_test_grad_descendant=find(F<0);
        valide_F=P(conformite_test_grad_descendant+imax,:);
        
        if length(valide_F)==length(F)
            disp('Data are good according to the gradient test on the descending part');
        else
            warning('There is (are) artefact(s) on the descending part');
            erreur_test_grad_descendant=find(F>=0);
            erreur_F=P(erreur_test_grad_descendant+imax,:);
            Z=P;
            Z(erreur_test_grad_descendant+imax,:)=NaN;
            
            figure(4)
            clf;
            plot(P(imax:o,1),P(imax:o,4),'+b');
            datetick('x',13);
            hold on;
            plot(P(imax:o,1),Z(imax:o,4),'r');
            datetick('x',13);
            title('Artefact(s) on the descending part');
            hgsave(4,[Nomrhodamine, '_Descending_part.fig']);
            
            [f1,f2]=size(erreur_F);
            
            h=1;
            
            while h<=f1
                inf_h=erreur_test_grad_descendant(h)-1;
                sup_h=erreur_test_grad_descendant(h)+1;
                inf_eventuel=find(conformite_test_grad_descendant==inf_h);
                sup_eventuel=find(conformite_test_grad_descendant==sup_h);
                exist_inf_h=isempty(inf_eventuel);
                exist_sup_h=isempty(sup_eventuel);
                
                if exist_inf_h + exist_sup_h==0
                    indice_h=erreur_test_grad_descendant(h)+imax;
                    disp(['For the time stamp : ',datestr(P(indice_h,1))]);
                    ans=input('Would you like to take the mean for the missing value (yes or no)? ','s');
                    	if ans=='yes'
                            P(indice_h,4)=(P(indice_h+1,4)+P(indice_h-1,4))/2;
                        else
                            rec=input('Enter the value here: ');
                            P(indice_h,4)=rec;
                        end
                        
                h=h+1;
                        
                else
                    warning('The artefact occured for several time steps in a row');
                    indice_h=erreur_test_grad_descendant(h);
                    F_inf=F(1:indice_h-1,:);
                    f_inf=length(F_inf);
                    conformite_test_grad_descendant_F_inf=find(F_inf<0);
                    a=max(conformite_test_grad_descendant_F_inf);
                    F_sup=F(indice_h+1:length(F),:); 
                    f_sup=length(F_sup);
                    conformite_test_grad_descendant_F_sup=find(F_sup<0);
                    b=min(conformite_test_grad_descendant_F_sup);
                    if isempty(b)==1
                        warning('Values are doubtful until the end');
                        ans_tf=input('Would you like to change the value at the end (yes or no)','s');
                        if ans_tf=='yes'
                            rec_tf=input(['Please enter the value for: ',datestr(P(length(F)+imax,1)),' ',num2str(P(length(F)+imax,4)),': ']);
                            P(length(F)+imax,4)=rec_tf;
                        else
                            if ans_tf=='non'
                                P(length(F)+imax,4)=P(length(F)+imax,4);
                            end
                        end
                    else
                        disp(['Data between ',datestr(P(a+imax,1)),' & ',datestr(P(indice_h+imax+b,1)),' are doubtful']);
                        disp('Please enter the value for the following the stamps: ');
                        for e=a:indice_h+b
                            disp([datestr(P(e+imax,1)),' : ',num2str(P(e+imax,4))]);
                            val_e=input('Value for the time stamp: ');
                            P(e+imax-1,4)=val_e;
                        end
                    end

                h=h+b+1;
                    
                end
                
            end

            close 4;
        
        end



    disp('Data are validated on the descending part');
    
    clear F & Z & conformite_test_grad_descendant & valide_F & erreur_test_grad_descendant & f1 & f2 & h & inf_h & sup_h & inf_eventuel & sup_eventuel & exist_inf_h & exist_sup_h & indice_h & ans & rec & F_inf & f_inf & conformite_test_grad_descendant_inf & a & F_sup & f_sup & conformite_test_grad_descendant_sup & b & e & val_e; 
    
    disp('Data have been validated according to the gradient test');
    
    %----------------------------------------------------------------------
    % Saving data
     
    J=A;
    x=find(J(:,1)==td);
    y=find(J(:,1)==tf);
    J(x:y,:)=P;
    J(1:x-1,4)=J(1:x-1,4)-Cbruitf;
    J(1:x-1,5)=sqrt(J(1:x-1,5).^2+uCbruitf^2);
    J(y+1:length(A),4)=J(y+1:length(A),4)-Cbruitf;
    J(y+1:length(A),5)=sqrt(J(y+1:length(A),5).^2+uCbruitf^2);
   
%__________________________________________________________________________
% DISCHARGE AND UNCERTAINTY CALCULATIONS

    %---------------------------------------------------------------------- 
    % Calculations
    
    P(:,4:5)=P(:,4:5).*10^-3;
    
    U(1,1)=10^-3*u_Cinj*(10^-9*v0/sum(P(:,4)));
    U(1,2)=10^-9*u_Vinj*(10^-3*c0/sum(P(:,4)));
    U(1,3)=sqrt(sum(P(:,5).^2))*m0/(dt*(sum(P(:,4)).^2));
    U(1,4)=sqrt(U(1,1)^2+U(1,2)^2+U(1,3)^2);

    
    Q=1000*m0/(dt*sum(P(:,4)));
    u_Q=2*U(1,4)*1000;
    z=u_Q/Q*100;
    
    disp(['The discharge (in l/s) measured with the tracer experiment is equal to: ',num2str(Q),'(l/s)']);
    disp(['The 95% confdence interval is equal to +  or -: ',num2str(u_Q),'(l/s)']);
    disp(['I.e. : ',num2str(z),' %']);
    
    R=[Q;u_Q/2];
    dlmwrite([Nomrhodamine, '_result-tracer.dat'], R);
    