function Y = uTypeB(Z,A,chain,alpha,MatCor,varargin)
% (C) JLBK, 08/2018
% Five mandatory inputs
% 1 Z: matrix of time varying quantities Zi
% 2 A: matrix of constant quantities Aj
% 3 chaine: string representing the measurement process, based on
%   quantities Zi and Aj
% 4 alpha: level of probability for the coverage interval
% 5 MatCor: matrix of correlation between Zi and Aj quantities
%   described in matrices Z and A
% Two optional inputs as varargin inputs:
% 1 NuZ: vector of degress of freedom of quantities Zi
% 2 NuA: vector of degress of freedom of quantities Aj
% Outputs matrix Y
% Y(:,1): best estimate y of the true value of Y
% Y(:,2): standard uncertainty in Y
% Y(:,3:4): boundaries of coverage interval with default value k for alpha
%   and an inifinite number degree of freedom
% Y(:,5:6): boundaries of coverage interval with value k calculated with nueff

% function Y = f(Z,A) derived from chaine
function y=fonctionMC(Z,A)
y=eval(chain);
end

% intermediate matrices for calculations
% matrix of quantity Z values 
M1=Z(:,1:2:size(Z,2)-1);
% matrix of standard uncertainties u(Z) values
M2=Z(:,2:2:size(Z,2));
% matrix of quantity A values 
P1=A(:,1:2:size(A,2)-1);
% matrix of standard uncertainties u(A) values
P2=A(:,2:2:size(A,2));
% epsilon value for 2nd order numerical approximations of sensitivity coefficients
% epsilon can't be equal to zero, so there is a test to avoid this zero value
% by replacing zeros values by the arbitrary value 1e-8.
 eps=min([min(max(M2,1e-8)) min(max(P2,1e-8))])/1e3;

% preallocation of Y
if nargin == 7 
    Y=zeros(size(M1,1),6);
else
    Y=zeros(size(M1,1),4);
end     

% scanning of the Z times series
for k=1:size(M1,1)
    % calculation of Y values
    Y(k,1)= fonctionMC(M1(k,:),P1);
    % calculation of Zi sensitivity coefficients
  for x= 1:size(M1,2)
    EPS = zeros(1,size(M1,2));
    EPS(1,x)=eps;
    ciZ(1,x)=(fonctionMC(M1(k,:)+EPS,P1)-fonctionMC(M1(k,:)-EPS,P1))/2/eps;
  end  
    % calculation of Aj sensitivity coefficients
  for a= 1:size(P1,2)
    EPS = zeros(1,size(P1,2));
    EPS(1,a)=eps;
    ciA(1,a)=(fonctionMC(M1(k,:),P1+EPS)-fonctionMC(M1(k,:),P1-EPS))/2/eps;
  end
% calculation of standard uncertainties u(Y)
Y(k,2)=sqrt([ciZ ciA]*((repmat([M2(k,:) P2],size([M2(k,:) P2],2),1).*repmat([M2(k,:) P2]',1,size([M2(k,:) P2],2))).*MatCor)*[ciZ ciA]');

% boundaries of coverage interval with default k value for alpha 
Y(k,3:4)=[Y(k,1)-tinv((1+alpha)/2,Inf)*Y(k,2) Y(k,1)+tinv((1+alpha)/2,Inf)*Y(k,2)];

% boundaries of coverage interval with k value calculated with nueff
% calculated only if NuZ and NuA are given as two optionnal function inputs
if nargin == 7 
    nueff=floor(power(Y(k,2),4)/sum(power([M2(k,:) P2].*[ciZ ciA],4)./[varargin{1} varargin{2}]));
    Y(k,5:6)=[Y(k,1)-tinv((1+alpha)/2,nueff)*Y(k,2) Y(k,1)+tinv((1+alpha)/2,nueff)*Y(k,2)];
end
end
end