function y=uTypeBsum(M)
% (c) JLBK 11/2016 - Revised 01/2018
% Calculates the sum of the vector V and its standard uncertainty u(V)
% by applying the Type B method under three different assumptions:
% - no correlation (r = 0)
% - full correlation (r = 1)
% - partial auto-correlation determined from the varigraph.
% The third assumption is the recommended approach. The two first
% assumptions are given for comparison.
% The input matrix M shall contain:
% - column 1: the values of the quantity V
% - column 2: the standard uncertainties of the quantity V.

function m = variograph(V)
% (C) JLBK 09/2016
% Calculates the variograph gamma(t) of the vector V
% and gives the number m of time steps beyond which the covariance
% becomes larger than the variance of the vector V.

% pre-allocation
gammat=zeros(size(V,1),1);
% calculation of gamma
for dec=1:size(V,1)-1
  gammat(dec,1)=1/2/(size(V,1)-dec).*sum((V(1+dec:size(V,1),1)-V(1:size(V,1)-dec,1)).^2);
end
% calculation of m
m=min(find(gammat-var(V)>=0))-1;
end


% calculation of the sum
y(1,1)=sum(M(:,1));
% standard uncertainty with no correlation
y(2,1)=sqrt(sum(M(:,2).^2));
% standard uncertainty with full correlation
y(3,1)=sum(M(:,2));
% standard uncertainty with variograph
% calculation of the horizon p of the variograph
p = variograph(M(:,1));
% calculation of the coefficients of correlation r(Xi+dec, Xi)
% between values of V with a time interval dec given as a number of time
% steps.
% pre-allocation
corrdec=zeros(p,1);
for dec=1:p
  corrdec(dec,1)=corr(M(dec+1:size(M,1),1),M(1:size(M,1)-dec,1));
end
% calculation of the double sum with partial auto-correlation
DoubleSomme=0;
for i=1:size(M,1)-1
  for j=i+1:min(size(M,1),i+p)
    DoubleSomme = DoubleSomme + 2*corrdec(j-i).*M(i,2).*M(j,2);
  end
end
% calculation of the standard uncertainty with partial auto-correlation
y(4,1)=sqrt(sum(M(:,2).^2) + DoubleSomme);
end