clear all
close all
% Matlab code to determing the minimal and maximal uncertainty in the measuring device given a
% signal to reproduce and a sampling interval based on frequency- and time
% domain analysis respectively.
% read Hydrograph data (an equidistant time series is expected here)
load('IWA_hydro.mat');
n=size(x1,1);
% make sure only an even number of datapoints is used
if mod(n,2)==1
    n=n-1;
end
y=x1(1:n);
dt=1; %timestep is 1 minute
% set value for gamma (see section 6.2.5.2)
quest=['Value for gamma: '];
gamma=input(quest);
% set signal to noise ratio
quest=['Signal to Noise Ratio: '];
SNR=input(quest);
% set time vector
t=(0:n-1)*dt;
% time domain analysis for establishing lower limit sampling frequency
% determining the autocorrelation function
u=autocorr(y);
%define the time lag vector
for i=1:1:size(u,1)
    lag_time(i)=i-1;
end
% plot the autocorrelation function
h=figure;
plot(lag_time,u,'-k','LineWidth',2);
grid on
xlabel('lag [min]','FontSize',14);
ylabel('\rho [-]','FontSize',14);
title('Normaslised Autocorrelation function hydrograph','FontSize',14);
% determine the standarddeviation for the signals y and yy
sigma_process=std(y);
disp(strcat('Standard deviation of the process= ',num2str(sigma_process),' m'));
% maximum allowable value for the measuring uncertainty based on time domain
% analysis
mse=0.5*sigma_process^2*(3+u(3)-4*u(2));
sigma_max=sqrt(mse/gamma);
disp(strcat('mse interploation= ',num2str(mse)));
disp(strcat('Upper limit measuring uncertainty (1 std)=',num2str(sigma_max),' m    time domain analysis'));
% plot the hydrograph
h=figure;
plot(y,'-ko','LineWidth',2);
xlabel('Time [minutes]','FontSize',14);
ylabel('Waterlevel [m]','FontSize',14);
grid on
title([' Example hydrograph, \sigma_p= ',num2str(sigma_process),' [m]'],'FontSize',14);
% Sampling frequency in the imported signal (once per minute) convert to
% hourly rate Fs
Fs=60/dt;
% frequency domain analysis using the matlab Fast Fourier Transform function fft
Y=fft(y);
% set frequency vector
f=2*Fs*(0:(n/2))/n;
% set the complex magnitude for each frequency
Pyy=abs(Y/n);
% reduce the length of the magnitude vector to single sided spectrum
P=Pyy(1:n/2+1);
% correct result for two sided spectrum to single sided spectrum
P(2:end-1)=2*P(2:end-1);
% plot spectrum
h=figure;
plot(f,log10(abs(P)),'-k','LineWidth',2);
hold on
min_a=min(P);
% plot the level of minimal power in the signal in the specturm inclusinf
% the SNR
min_1=min_a/SNR;
plot([0,f(n/2)],[log10(min_1),log10(min_1)],'-b','LineWidth',2);
hold on
xlabel('Frequency [h^-^1]');
legend('Spectrum',strcat('\sigma_m at SNR=',num2str(SNR)));
ylabel('log^1^0|P(f)|','FontSize',14);
title('Full spectrum','FontSize',14);
grid on
sigma_min=sqrt(min_1);
disp(strcat('Lower limit measuring uncertainty (1 std)=',num2str(sigma_min),' m  frequency domain analysis'));